from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from google.rpc.status_pb2 import Status
from typing_extensions import Literal

from google.ads.googleads.v20.common.types.criteria import KeywordInfo
from google.ads.googleads.v20.common.types.extensions import (
    CallFeedItem,
    CalloutFeedItem,
    SitelinkFeedItem,
)
from google.ads.googleads.v20.enums.types.ad_group_type import AdGroupTypeEnum
from google.ads.googleads.v20.enums.types.advertising_channel_type import (
    AdvertisingChannelTypeEnum,
)
from google.ads.googleads.v20.enums.types.bidding_strategy_type import (
    BiddingStrategyTypeEnum,
)
from google.ads.googleads.v20.enums.types.conversion_tracking_status_enum import (
    ConversionTrackingStatusEnum,
)
from google.ads.googleads.v20.enums.types.keyword_match_type import KeywordMatchTypeEnum
from google.ads.googleads.v20.enums.types.recommendation_type import (
    RecommendationTypeEnum,
)
from google.ads.googleads.v20.enums.types.target_impression_share_location import (
    TargetImpressionShareLocationEnum,
)
from google.ads.googleads.v20.resources.types.ad import Ad
from google.ads.googleads.v20.resources.types.asset import Asset
from google.ads.googleads.v20.resources.types.recommendation import Recommendation

_M = TypeVar("_M")

class ApplyRecommendationOperation(proto.Message):
    class AdAssetApplyParameters(proto.Message):
        class ApplyScope(proto.Enum):
            UNSPECIFIED = 0
            UNKNOWN = 1
            CUSTOMER = 2
            CAMPAIGN = 3

        new_assets: MutableSequence[Asset]
        existing_assets: MutableSequence[str]
        scope: ApplyRecommendationOperation.AdAssetApplyParameters.ApplyScope
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            new_assets: MutableSequence[Asset] = ...,
            existing_assets: MutableSequence[str] = ...,
            scope: ApplyRecommendationOperation.AdAssetApplyParameters.ApplyScope = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["new_assets", "existing_assets", "scope"]
        ) -> bool: ...

    class CallAssetParameters(proto.Message):
        ad_asset_apply_parameters: ApplyRecommendationOperation.AdAssetApplyParameters
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            ad_asset_apply_parameters: ApplyRecommendationOperation.AdAssetApplyParameters = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["ad_asset_apply_parameters"]
        ) -> bool: ...

    class CallExtensionParameters(proto.Message):
        call_extensions: MutableSequence[CallFeedItem]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            call_extensions: MutableSequence[CallFeedItem] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["call_extensions"]
        ) -> bool: ...

    class CalloutAssetParameters(proto.Message):
        ad_asset_apply_parameters: ApplyRecommendationOperation.AdAssetApplyParameters
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            ad_asset_apply_parameters: ApplyRecommendationOperation.AdAssetApplyParameters = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["ad_asset_apply_parameters"]
        ) -> bool: ...

    class CalloutExtensionParameters(proto.Message):
        callout_extensions: MutableSequence[CalloutFeedItem]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            callout_extensions: MutableSequence[CalloutFeedItem] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["callout_extensions"]
        ) -> bool: ...

    class CampaignBudgetParameters(proto.Message):
        new_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            new_budget_amount_micros: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["new_budget_amount_micros"]
        ) -> bool: ...

    class ForecastingSetTargetCpaParameters(proto.Message):
        target_cpa_micros: int
        campaign_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            target_cpa_micros: int = ...,
            campaign_budget_amount_micros: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["target_cpa_micros", "campaign_budget_amount_micros"]
        ) -> bool: ...

    class ForecastingSetTargetRoasParameters(proto.Message):
        target_roas: float
        campaign_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            target_roas: float = ...,
            campaign_budget_amount_micros: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["target_roas", "campaign_budget_amount_micros"]
        ) -> bool: ...

    class KeywordParameters(proto.Message):
        ad_group: str
        match_type: KeywordMatchTypeEnum.KeywordMatchType
        cpc_bid_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            ad_group: str = ...,
            match_type: KeywordMatchTypeEnum.KeywordMatchType = ...,
            cpc_bid_micros: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["ad_group", "match_type", "cpc_bid_micros"]
        ) -> bool: ...

    class LeadFormAssetParameters(proto.Message):
        ad_asset_apply_parameters: ApplyRecommendationOperation.AdAssetApplyParameters
        set_submit_lead_form_asset_campaign_goal: bool
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            ad_asset_apply_parameters: ApplyRecommendationOperation.AdAssetApplyParameters = ...,
            set_submit_lead_form_asset_campaign_goal: bool = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self,
            key: Literal[
                "ad_asset_apply_parameters", "set_submit_lead_form_asset_campaign_goal"
            ],
        ) -> bool: ...

    class LowerTargetRoasParameters(proto.Message):
        target_roas_multiplier: float
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            target_roas_multiplier: float = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["target_roas_multiplier"]
        ) -> bool: ...

    class MoveUnusedBudgetParameters(proto.Message):
        budget_micros_to_move: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            budget_micros_to_move: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["budget_micros_to_move"]
        ) -> bool: ...

    class RaiseTargetCpaBidTooLowParameters(proto.Message):
        target_multiplier: float
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            target_multiplier: float = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["target_multiplier"]
        ) -> bool: ...

    class RaiseTargetCpaParameters(proto.Message):
        target_cpa_multiplier: float
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            target_cpa_multiplier: float = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["target_cpa_multiplier"]
        ) -> bool: ...

    class ResponsiveSearchAdAssetParameters(proto.Message):
        updated_ad: Ad
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            updated_ad: Ad = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["updated_ad"]
        ) -> bool: ...

    class ResponsiveSearchAdImproveAdStrengthParameters(proto.Message):
        updated_ad: Ad
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            updated_ad: Ad = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["updated_ad"]
        ) -> bool: ...

    class ResponsiveSearchAdParameters(proto.Message):
        ad: Ad
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            ad: Ad = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["ad"]
        ) -> bool: ...

    class SitelinkAssetParameters(proto.Message):
        ad_asset_apply_parameters: ApplyRecommendationOperation.AdAssetApplyParameters
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            ad_asset_apply_parameters: ApplyRecommendationOperation.AdAssetApplyParameters = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["ad_asset_apply_parameters"]
        ) -> bool: ...

    class SitelinkExtensionParameters(proto.Message):
        sitelink_extensions: MutableSequence[SitelinkFeedItem]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            sitelink_extensions: MutableSequence[SitelinkFeedItem] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["sitelink_extensions"]
        ) -> bool: ...

    class TargetCpaOptInParameters(proto.Message):
        target_cpa_micros: int
        new_campaign_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            target_cpa_micros: int = ...,
            new_campaign_budget_amount_micros: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["target_cpa_micros", "new_campaign_budget_amount_micros"]
        ) -> bool: ...

    class TargetRoasOptInParameters(proto.Message):
        target_roas: float
        new_campaign_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            target_roas: float = ...,
            new_campaign_budget_amount_micros: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["target_roas", "new_campaign_budget_amount_micros"]
        ) -> bool: ...

    class TextAdParameters(proto.Message):
        ad: Ad
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            ad: Ad = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["ad"]
        ) -> bool: ...

    class UseBroadMatchKeywordParameters(proto.Message):
        new_budget_amount_micros: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            new_budget_amount_micros: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["new_budget_amount_micros"]
        ) -> bool: ...

    resource_name: str
    campaign_budget: ApplyRecommendationOperation.CampaignBudgetParameters
    text_ad: ApplyRecommendationOperation.TextAdParameters
    keyword: ApplyRecommendationOperation.KeywordParameters
    target_cpa_opt_in: ApplyRecommendationOperation.TargetCpaOptInParameters
    target_roas_opt_in: ApplyRecommendationOperation.TargetRoasOptInParameters
    callout_extension: ApplyRecommendationOperation.CalloutExtensionParameters
    call_extension: ApplyRecommendationOperation.CallExtensionParameters
    sitelink_extension: ApplyRecommendationOperation.SitelinkExtensionParameters
    move_unused_budget: ApplyRecommendationOperation.MoveUnusedBudgetParameters
    responsive_search_ad: ApplyRecommendationOperation.ResponsiveSearchAdParameters
    use_broad_match_keyword: ApplyRecommendationOperation.UseBroadMatchKeywordParameters
    responsive_search_ad_asset: (
        ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters
    )
    responsive_search_ad_improve_ad_strength: (
        ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters
    )
    raise_target_cpa_bid_too_low: (
        ApplyRecommendationOperation.RaiseTargetCpaBidTooLowParameters
    )
    forecasting_set_target_roas: (
        ApplyRecommendationOperation.ForecastingSetTargetRoasParameters
    )
    callout_asset: ApplyRecommendationOperation.CalloutAssetParameters
    call_asset: ApplyRecommendationOperation.CallAssetParameters
    sitelink_asset: ApplyRecommendationOperation.SitelinkAssetParameters
    raise_target_cpa: ApplyRecommendationOperation.RaiseTargetCpaParameters
    lower_target_roas: ApplyRecommendationOperation.LowerTargetRoasParameters
    forecasting_set_target_cpa: (
        ApplyRecommendationOperation.ForecastingSetTargetCpaParameters
    )
    set_target_cpa: ApplyRecommendationOperation.ForecastingSetTargetCpaParameters
    set_target_roas: ApplyRecommendationOperation.ForecastingSetTargetRoasParameters
    lead_form_asset: ApplyRecommendationOperation.LeadFormAssetParameters
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
        campaign_budget: ApplyRecommendationOperation.CampaignBudgetParameters = ...,
        text_ad: ApplyRecommendationOperation.TextAdParameters = ...,
        keyword: ApplyRecommendationOperation.KeywordParameters = ...,
        target_cpa_opt_in: ApplyRecommendationOperation.TargetCpaOptInParameters = ...,
        target_roas_opt_in: ApplyRecommendationOperation.TargetRoasOptInParameters = ...,
        callout_extension: ApplyRecommendationOperation.CalloutExtensionParameters = ...,
        call_extension: ApplyRecommendationOperation.CallExtensionParameters = ...,
        sitelink_extension: ApplyRecommendationOperation.SitelinkExtensionParameters = ...,
        move_unused_budget: ApplyRecommendationOperation.MoveUnusedBudgetParameters = ...,
        responsive_search_ad: ApplyRecommendationOperation.ResponsiveSearchAdParameters = ...,
        use_broad_match_keyword: ApplyRecommendationOperation.UseBroadMatchKeywordParameters = ...,
        responsive_search_ad_asset: ApplyRecommendationOperation.ResponsiveSearchAdAssetParameters = ...,
        responsive_search_ad_improve_ad_strength: ApplyRecommendationOperation.ResponsiveSearchAdImproveAdStrengthParameters = ...,
        raise_target_cpa_bid_too_low: ApplyRecommendationOperation.RaiseTargetCpaBidTooLowParameters = ...,
        forecasting_set_target_roas: ApplyRecommendationOperation.ForecastingSetTargetRoasParameters = ...,
        callout_asset: ApplyRecommendationOperation.CalloutAssetParameters = ...,
        call_asset: ApplyRecommendationOperation.CallAssetParameters = ...,
        sitelink_asset: ApplyRecommendationOperation.SitelinkAssetParameters = ...,
        raise_target_cpa: ApplyRecommendationOperation.RaiseTargetCpaParameters = ...,
        lower_target_roas: ApplyRecommendationOperation.LowerTargetRoasParameters = ...,
        forecasting_set_target_cpa: ApplyRecommendationOperation.ForecastingSetTargetCpaParameters = ...,
        set_target_cpa: ApplyRecommendationOperation.ForecastingSetTargetCpaParameters = ...,
        set_target_roas: ApplyRecommendationOperation.ForecastingSetTargetRoasParameters = ...,
        lead_form_asset: ApplyRecommendationOperation.LeadFormAssetParameters = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "resource_name",
            "campaign_budget",
            "text_ad",
            "keyword",
            "target_cpa_opt_in",
            "target_roas_opt_in",
            "callout_extension",
            "call_extension",
            "sitelink_extension",
            "move_unused_budget",
            "responsive_search_ad",
            "use_broad_match_keyword",
            "responsive_search_ad_asset",
            "responsive_search_ad_improve_ad_strength",
            "raise_target_cpa_bid_too_low",
            "forecasting_set_target_roas",
            "callout_asset",
            "call_asset",
            "sitelink_asset",
            "raise_target_cpa",
            "lower_target_roas",
            "forecasting_set_target_cpa",
            "set_target_cpa",
            "set_target_roas",
            "lead_form_asset",
        ],
    ) -> bool: ...

class ApplyRecommendationRequest(proto.Message):
    customer_id: str
    operations: MutableSequence[ApplyRecommendationOperation]
    partial_failure: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        operations: MutableSequence[ApplyRecommendationOperation] = ...,
        partial_failure: bool = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["customer_id", "operations", "partial_failure"]
    ) -> bool: ...

class ApplyRecommendationResponse(proto.Message):
    results: MutableSequence[ApplyRecommendationResult]
    partial_failure_error: Status
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        results: MutableSequence[ApplyRecommendationResult] = ...,
        partial_failure_error: Status = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["results", "partial_failure_error"]
    ) -> bool: ...

class ApplyRecommendationResult(proto.Message):
    resource_name: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["resource_name"]
    ) -> bool: ...

class DismissRecommendationRequest(proto.Message):
    class DismissRecommendationOperation(proto.Message):
        resource_name: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            resource_name: str = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["resource_name"]
        ) -> bool: ...

    customer_id: str
    operations: MutableSequence[
        DismissRecommendationRequest.DismissRecommendationOperation
    ]
    partial_failure: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        operations: MutableSequence[
            DismissRecommendationRequest.DismissRecommendationOperation
        ] = ...,
        partial_failure: bool = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["customer_id", "operations", "partial_failure"]
    ) -> bool: ...

class DismissRecommendationResponse(proto.Message):
    class DismissRecommendationResult(proto.Message):
        resource_name: str
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            resource_name: str = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["resource_name"]
        ) -> bool: ...

    results: MutableSequence[DismissRecommendationResponse.DismissRecommendationResult]
    partial_failure_error: Status
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        results: MutableSequence[
            DismissRecommendationResponse.DismissRecommendationResult
        ] = ...,
        partial_failure_error: Status = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["results", "partial_failure_error"]
    ) -> bool: ...

class GenerateRecommendationsRequest(proto.Message):
    class AdGroupInfo(proto.Message):
        ad_group_type: AdGroupTypeEnum.AdGroupType
        keywords: MutableSequence[KeywordInfo]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            ad_group_type: AdGroupTypeEnum.AdGroupType = ...,
            keywords: MutableSequence[KeywordInfo] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["ad_group_type", "keywords"]
        ) -> bool: ...

    class AssetGroupInfo(proto.Message):
        final_url: str
        headline: MutableSequence[str]
        description: MutableSequence[str]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            final_url: str = ...,
            headline: MutableSequence[str] = ...,
            description: MutableSequence[str] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["final_url", "headline", "description"]
        ) -> bool: ...

    class BiddingInfo(proto.Message):
        bidding_strategy_type: BiddingStrategyTypeEnum.BiddingStrategyType
        target_cpa_micros: int
        target_roas: float
        target_impression_share_info: (
            GenerateRecommendationsRequest.TargetImpressionShareInfo
        )
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            bidding_strategy_type: BiddingStrategyTypeEnum.BiddingStrategyType = ...,
            target_cpa_micros: int = ...,
            target_roas: float = ...,
            target_impression_share_info: GenerateRecommendationsRequest.TargetImpressionShareInfo = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self,
            key: Literal[
                "bidding_strategy_type",
                "target_cpa_micros",
                "target_roas",
                "target_impression_share_info",
            ],
        ) -> bool: ...

    class BudgetInfo(proto.Message):
        current_budget: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            current_budget: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["current_budget"]
        ) -> bool: ...

    class SeedInfo(proto.Message):
        url_seed: str
        keyword_seeds: MutableSequence[str]
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            url_seed: str = ...,
            keyword_seeds: MutableSequence[str] = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self, key: Literal["url_seed", "keyword_seeds"]
        ) -> bool: ...

    class TargetImpressionShareInfo(proto.Message):
        location: TargetImpressionShareLocationEnum.TargetImpressionShareLocation
        target_impression_share_micros: int
        max_cpc_bid_ceiling: int
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            location: TargetImpressionShareLocationEnum.TargetImpressionShareLocation = ...,
            target_impression_share_micros: int = ...,
            max_cpc_bid_ceiling: int = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self,
            key: Literal[
                "location", "target_impression_share_micros", "max_cpc_bid_ceiling"
            ],
        ) -> bool: ...

    customer_id: str
    recommendation_types: MutableSequence[RecommendationTypeEnum.RecommendationType]
    advertising_channel_type: AdvertisingChannelTypeEnum.AdvertisingChannelType
    campaign_sitelink_count: int
    conversion_tracking_status: ConversionTrackingStatusEnum.ConversionTrackingStatus
    bidding_info: GenerateRecommendationsRequest.BiddingInfo
    ad_group_info: MutableSequence[GenerateRecommendationsRequest.AdGroupInfo]
    seed_info: GenerateRecommendationsRequest.SeedInfo
    budget_info: GenerateRecommendationsRequest.BudgetInfo
    campaign_image_asset_count: int
    campaign_call_asset_count: int
    country_codes: MutableSequence[str]
    language_codes: MutableSequence[str]
    positive_locations_ids: MutableSequence[int]
    negative_locations_ids: MutableSequence[int]
    asset_group_info: MutableSequence[GenerateRecommendationsRequest.AssetGroupInfo]
    target_partner_search_network: bool
    target_content_network: bool
    merchant_center_account_id: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        recommendation_types: MutableSequence[
            RecommendationTypeEnum.RecommendationType
        ] = ...,
        advertising_channel_type: AdvertisingChannelTypeEnum.AdvertisingChannelType = ...,
        campaign_sitelink_count: int = ...,
        conversion_tracking_status: ConversionTrackingStatusEnum.ConversionTrackingStatus = ...,
        bidding_info: GenerateRecommendationsRequest.BiddingInfo = ...,
        ad_group_info: MutableSequence[
            GenerateRecommendationsRequest.AdGroupInfo
        ] = ...,
        seed_info: GenerateRecommendationsRequest.SeedInfo = ...,
        budget_info: GenerateRecommendationsRequest.BudgetInfo = ...,
        campaign_image_asset_count: int = ...,
        campaign_call_asset_count: int = ...,
        country_codes: MutableSequence[str] = ...,
        language_codes: MutableSequence[str] = ...,
        positive_locations_ids: MutableSequence[int] = ...,
        negative_locations_ids: MutableSequence[int] = ...,
        asset_group_info: MutableSequence[
            GenerateRecommendationsRequest.AssetGroupInfo
        ] = ...,
        target_partner_search_network: bool = ...,
        target_content_network: bool = ...,
        merchant_center_account_id: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "customer_id",
            "recommendation_types",
            "advertising_channel_type",
            "campaign_sitelink_count",
            "conversion_tracking_status",
            "bidding_info",
            "ad_group_info",
            "seed_info",
            "budget_info",
            "campaign_image_asset_count",
            "campaign_call_asset_count",
            "country_codes",
            "language_codes",
            "positive_locations_ids",
            "negative_locations_ids",
            "asset_group_info",
            "target_partner_search_network",
            "target_content_network",
            "merchant_center_account_id",
        ],
    ) -> bool: ...

class GenerateRecommendationsResponse(proto.Message):
    recommendations: MutableSequence[Recommendation]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        recommendations: MutableSequence[Recommendation] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["recommendations"]
    ) -> bool: ...



from .async_client import (
    CampaignSharedSetServiceAsyncClient as CampaignSharedSetServiceAsyncClient,
)
from .client import CampaignSharedSetServiceClient as CampaignSharedSetServiceClient

__all__ = ["CampaignSharedSetServiceClient", "CampaignSharedSetServiceAsyncClient"]



from .async_client import (
    KeywordPlanIdeaServiceAsyncClient as KeywordPlanIdeaServiceAsyncClient,
)
from .client import KeywordPlanIdeaServiceClient as KeywordPlanIdeaServiceClient

__all__ = ["KeywordPlanIdeaServiceClient", "KeywordPlanIdeaServiceAsyncClient"]



from .async_client import (
    BillingSetupServiceAsyncClient as BillingSetupServiceAsyncClient,
)
from .client import BillingSetupServiceClient as BillingSetupServiceClient

__all__ = ["BillingSetupServiceClient", "BillingSetupServiceAsyncClient"]



from collections.abc import Mapping
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

_M = TypeVar("_M")

class UserListCustomerTypeCategoryEnum(proto.Message):
    class UserListCustomerTypeCategory(proto.Enum):
        UNSPECIFIED = 0
        UNKNOWN = 1
        ALL_CUSTOMERS = 2
        PURCHASERS = 3
        HIGH_VALUE_CUSTOMERS = 4
        DISENGAGED_CUSTOMERS = 5
        QUALIFIED_LEADS = 6
        CONVERTED_LEADS = 7
        PAID_SUBSCRIBERS = 8
        LOYALTY_SIGN_UPS = 9
        CART_ABANDONERS = 10
        LOYALTY_TIER_1_MEMBERS = 11
        LOYALTY_TIER_2_MEMBERS = 12
        LOYALTY_TIER_3_MEMBERS = 13
        LOYALTY_TIER_4_MEMBERS = 14
        LOYALTY_TIER_5_MEMBERS = 15
        LOYALTY_TIER_6_MEMBERS = 16
        LOYALTY_TIER_7_MEMBERS = 17

    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: NoReturn
    ) -> bool: ...



from .base import (
    CustomerSkAdNetworkConversionValueSchemaServiceTransport as CustomerSkAdNetworkConversionValueSchemaServiceTransport,
)
from .grpc import (
    CustomerSkAdNetworkConversionValueSchemaServiceGrpcTransport as CustomerSkAdNetworkConversionValueSchemaServiceGrpcTransport,
)
from .grpc_asyncio import (
    CustomerSkAdNetworkConversionValueSchemaServiceGrpcAsyncIOTransport as CustomerSkAdNetworkConversionValueSchemaServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CustomerSkAdNetworkConversionValueSchemaServiceTransport",
    "CustomerSkAdNetworkConversionValueSchemaServiceGrpcTransport",
    "CustomerSkAdNetworkConversionValueSchemaServiceGrpcAsyncIOTransport",
]



from .async_client import (
    AutomaticallyCreatedAssetRemovalServiceAsyncClient as AutomaticallyCreatedAssetRemovalServiceAsyncClient,
)
from .client import (
    AutomaticallyCreatedAssetRemovalServiceClient as AutomaticallyCreatedAssetRemovalServiceClient,
)

__all__ = [
    "AutomaticallyCreatedAssetRemovalServiceClient",
    "AutomaticallyCreatedAssetRemovalServiceAsyncClient",
]



from .async_client import AccountLinkServiceAsyncClient as AccountLinkServiceAsyncClient
from .client import AccountLinkServiceClient as AccountLinkServiceClient

__all__ = ["AccountLinkServiceClient", "AccountLinkServiceAsyncClient"]



from .async_client import (
    ProductLinkInvitationServiceAsyncClient as ProductLinkInvitationServiceAsyncClient,
)
from .client import (
    ProductLinkInvitationServiceClient as ProductLinkInvitationServiceClient,
)

__all__ = [
    "ProductLinkInvitationServiceClient",
    "ProductLinkInvitationServiceAsyncClient",
]



from .base import AssetGenerationServiceTransport as AssetGenerationServiceTransport
from .grpc import (
    AssetGenerationServiceGrpcTransport as AssetGenerationServiceGrpcTransport,
)
from .grpc_asyncio import (
    AssetGenerationServiceGrpcAsyncIOTransport as AssetGenerationServiceGrpcAsyncIOTransport,
)

__all__ = [
    "AssetGenerationServiceTransport",
    "AssetGenerationServiceGrpcTransport",
    "AssetGenerationServiceGrpcAsyncIOTransport",
]



from .async_client import (
    KeywordThemeConstantServiceAsyncClient as KeywordThemeConstantServiceAsyncClient,
)
from .client import (
    KeywordThemeConstantServiceClient as KeywordThemeConstantServiceClient,
)

__all__ = [
    "KeywordThemeConstantServiceClient",
    "KeywordThemeConstantServiceAsyncClient",
]



from .base import ExperimentArmServiceTransport as ExperimentArmServiceTransport
from .grpc import ExperimentArmServiceGrpcTransport as ExperimentArmServiceGrpcTransport
from .grpc_asyncio import (
    ExperimentArmServiceGrpcAsyncIOTransport as ExperimentArmServiceGrpcAsyncIOTransport,
)

__all__ = [
    "ExperimentArmServiceTransport",
    "ExperimentArmServiceGrpcTransport",
    "ExperimentArmServiceGrpcAsyncIOTransport",
]



from .async_client import (
    ConversionValueRuleSetServiceAsyncClient as ConversionValueRuleSetServiceAsyncClient,
)
from .client import (
    ConversionValueRuleSetServiceClient as ConversionValueRuleSetServiceClient,
)

__all__ = [
    "ConversionValueRuleSetServiceClient",
    "ConversionValueRuleSetServiceAsyncClient",
]



from .base import CustomerAssetSetServiceTransport as CustomerAssetSetServiceTransport
from .grpc import (
    CustomerAssetSetServiceGrpcTransport as CustomerAssetSetServiceGrpcTransport,
)
from .grpc_asyncio import (
    CustomerAssetSetServiceGrpcAsyncIOTransport as CustomerAssetSetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "CustomerAssetSetServiceTransport",
    "CustomerAssetSetServiceGrpcTransport",
    "CustomerAssetSetServiceGrpcAsyncIOTransport",
]



from .base import AssetServiceTransport as AssetServiceTransport
from .grpc import AssetServiceGrpcTransport as AssetServiceGrpcTransport
from .grpc_asyncio import (
    AssetServiceGrpcAsyncIOTransport as AssetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "AssetServiceTransport",
    "AssetServiceGrpcTransport",
    "AssetServiceGrpcAsyncIOTransport",
]



from .async_client import (
    AdGroupLabelServiceAsyncClient as AdGroupLabelServiceAsyncClient,
)
from .client import AdGroupLabelServiceClient as AdGroupLabelServiceClient

__all__ = ["AdGroupLabelServiceClient", "AdGroupLabelServiceAsyncClient"]



from .async_client import (
    AssetGroupListingGroupFilterServiceAsyncClient as AssetGroupListingGroupFilterServiceAsyncClient,
)
from .client import (
    AssetGroupListingGroupFilterServiceClient as AssetGroupListingGroupFilterServiceClient,
)

__all__ = [
    "AssetGroupListingGroupFilterServiceClient",
    "AssetGroupListingGroupFilterServiceAsyncClient",
]



from .async_client import AssetGroupServiceAsyncClient as AssetGroupServiceAsyncClient
from .client import AssetGroupServiceClient as AssetGroupServiceClient

__all__ = ["AssetGroupServiceClient", "AssetGroupServiceAsyncClient"]



from .async_client import (
    AdGroupAssetSetServiceAsyncClient as AdGroupAssetSetServiceAsyncClient,
)
from .client import AdGroupAssetSetServiceClient as AdGroupAssetSetServiceClient

__all__ = ["AdGroupAssetSetServiceClient", "AdGroupAssetSetServiceAsyncClient"]



from .base import AssetSetServiceTransport as AssetSetServiceTransport
from .grpc import AssetSetServiceGrpcTransport as AssetSetServiceGrpcTransport
from .grpc_asyncio import (
    AssetSetServiceGrpcAsyncIOTransport as AssetSetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "AssetSetServiceTransport",
    "AssetSetServiceGrpcTransport",
    "AssetSetServiceGrpcAsyncIOTransport",
]



from .base import (
    TravelAssetSuggestionServiceTransport as TravelAssetSuggestionServiceTransport,
)
from .grpc import (
    TravelAssetSuggestionServiceGrpcTransport as TravelAssetSuggestionServiceGrpcTransport,
)
from .grpc_asyncio import (
    TravelAssetSuggestionServiceGrpcAsyncIOTransport as TravelAssetSuggestionServiceGrpcAsyncIOTransport,
)

__all__ = [
    "TravelAssetSuggestionServiceTransport",
    "TravelAssetSuggestionServiceGrpcTransport",
    "TravelAssetSuggestionServiceGrpcAsyncIOTransport",
]



from .base import LabelServiceTransport as LabelServiceTransport
from .grpc import LabelServiceGrpcTransport as LabelServiceGrpcTransport
from .grpc_asyncio import (
    LabelServiceGrpcAsyncIOTransport as LabelServiceGrpcAsyncIOTransport,
)

__all__ = [
    "LabelServiceTransport",
    "LabelServiceGrpcTransport",
    "LabelServiceGrpcAsyncIOTransport",
]



from .base import RemarketingActionServiceTransport as RemarketingActionServiceTransport
from .grpc import (
    RemarketingActionServiceGrpcTransport as RemarketingActionServiceGrpcTransport,
)
from .grpc_asyncio import (
    RemarketingActionServiceGrpcAsyncIOTransport as RemarketingActionServiceGrpcAsyncIOTransport,
)

__all__ = [
    "RemarketingActionServiceTransport",
    "RemarketingActionServiceGrpcTransport",
    "RemarketingActionServiceGrpcAsyncIOTransport",
]



from .base import AdGroupAssetServiceTransport as AdGroupAssetServiceTransport
from .grpc import AdGroupAssetServiceGrpcTransport as AdGroupAssetServiceGrpcTransport
from .grpc_asyncio import (
    AdGroupAssetServiceGrpcAsyncIOTransport as AdGroupAssetServiceGrpcAsyncIOTransport,
)

__all__ = [
    "AdGroupAssetServiceTransport",
    "AdGroupAssetServiceGrpcTransport",
    "AdGroupAssetServiceGrpcAsyncIOTransport",
]



from typing import Callable, MutableSequence, Sequence

from _typeshed import Incomplete
from google.api_core import gapic_v1, retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials

from google.ads.googleads.v19.services.types import shared_criterion_service

from .transports.base import SharedCriterionServiceTransport

__all__ = ["SharedCriterionServiceAsyncClient"]

class SharedCriterionServiceAsyncClient:
    DEFAULT_ENDPOINT: Incomplete
    DEFAULT_MTLS_ENDPOINT: Incomplete
    mobile_app_category_constant_path: Incomplete
    parse_mobile_app_category_constant_path: Incomplete
    shared_criterion_path: Incomplete
    parse_shared_criterion_path: Incomplete
    shared_set_path: Incomplete
    parse_shared_set_path: Incomplete
    common_billing_account_path: Incomplete
    parse_common_billing_account_path: Incomplete
    common_folder_path: Incomplete
    parse_common_folder_path: Incomplete
    common_organization_path: Incomplete
    parse_common_organization_path: Incomplete
    common_project_path: Incomplete
    parse_common_project_path: Incomplete
    common_location_path: Incomplete
    parse_common_location_path: Incomplete
    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs): ...
    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs): ...
    from_service_account_json = from_service_account_file
    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: ClientOptions | None = None
    ): ...
    @property
    def transport(self) -> SharedCriterionServiceTransport: ...
    @property
    def api_endpoint(self): ...
    @property
    def universe_domain(self) -> str: ...
    get_transport_class: Incomplete
    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials | None = None,
        transport: str
        | SharedCriterionServiceTransport
        | Callable[..., SharedCriterionServiceTransport]
        | None = "grpc_asyncio",
        client_options: ClientOptions | None = None,
        client_info: gapic_v1.client_info.ClientInfo = ...,
    ) -> None: ...
    async def mutate_shared_criteria(
        self,
        request: shared_criterion_service.MutateSharedCriteriaRequest
        | dict
        | None = None,
        *,
        customer_id: str | None = None,
        operations: MutableSequence[shared_criterion_service.SharedCriterionOperation]
        | None = None,
        retry: retries.AsyncRetry | gapic_v1.method._MethodDefault = ...,
        timeout: float | object = ...,
        metadata: Sequence[tuple[str, str | bytes]] = (),
    ) -> shared_criterion_service.MutateSharedCriteriaResponse: ...
    async def __aenter__(self) -> SharedCriterionServiceAsyncClient: ...
    async def __aexit__(self, exc_type, exc, tb) -> None: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from typing_extensions import Literal

from google.ads.googleads.v19.common.types.criteria import (
    AgeRangeInfo,
    GenderInfo,
    IncomeRangeInfo,
    LocationInfo,
    ParentalStatusInfo,
    UserInterestInfo,
    YouTubeChannelInfo,
    YouTubeVideoInfo,
)
from google.ads.googleads.v19.enums.types.audience_insights_dimension import (
    AudienceInsightsDimensionEnum,
)
from google.ads.googleads.v19.enums.types.insights_knowledge_graph_entity_capabilities import (
    InsightsKnowledgeGraphEntityCapabilitiesEnum,
)

_M = TypeVar("_M")

class AudienceInsightsAttribute(proto.Message):
    age_range: AgeRangeInfo
    gender: GenderInfo
    location: LocationInfo
    user_interest: UserInterestInfo
    entity: AudienceInsightsEntity
    category: AudienceInsightsCategory
    dynamic_lineup: AudienceInsightsDynamicLineup
    parental_status: ParentalStatusInfo
    income_range: IncomeRangeInfo
    youtube_channel: YouTubeChannelInfo
    youtube_video: YouTubeVideoInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        age_range: AgeRangeInfo = ...,
        gender: GenderInfo = ...,
        location: LocationInfo = ...,
        user_interest: UserInterestInfo = ...,
        entity: AudienceInsightsEntity = ...,
        category: AudienceInsightsCategory = ...,
        dynamic_lineup: AudienceInsightsDynamicLineup = ...,
        parental_status: ParentalStatusInfo = ...,
        income_range: IncomeRangeInfo = ...,
        youtube_channel: YouTubeChannelInfo = ...,
        youtube_video: YouTubeVideoInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "age_range",
            "gender",
            "location",
            "user_interest",
            "entity",
            "category",
            "dynamic_lineup",
            "parental_status",
            "income_range",
            "youtube_channel",
            "youtube_video",
        ],
    ) -> bool: ...

class AudienceInsightsAttributeMetadata(proto.Message):
    dimension: AudienceInsightsDimensionEnum.AudienceInsightsDimension
    attribute: AudienceInsightsAttribute
    display_name: str
    display_info: str
    potential_youtube_reach: int
    subscriber_share: float
    viewer_share: float
    youtube_channel_metadata: YouTubeChannelAttributeMetadata
    youtube_video_metadata: YouTubeVideoAttributeMetadata
    dynamic_attribute_metadata: DynamicLineupAttributeMetadata
    location_attribute_metadata: LocationAttributeMetadata
    user_interest_attribute_metadata: UserInterestAttributeMetadata
    knowledge_graph_attribute_metadata: KnowledgeGraphAttributeMetadata
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        dimension: AudienceInsightsDimensionEnum.AudienceInsightsDimension = ...,
        attribute: AudienceInsightsAttribute = ...,
        display_name: str = ...,
        display_info: str = ...,
        potential_youtube_reach: int = ...,
        subscriber_share: float = ...,
        viewer_share: float = ...,
        youtube_channel_metadata: YouTubeChannelAttributeMetadata = ...,
        youtube_video_metadata: YouTubeVideoAttributeMetadata = ...,
        dynamic_attribute_metadata: DynamicLineupAttributeMetadata = ...,
        location_attribute_metadata: LocationAttributeMetadata = ...,
        user_interest_attribute_metadata: UserInterestAttributeMetadata = ...,
        knowledge_graph_attribute_metadata: KnowledgeGraphAttributeMetadata = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "dimension",
            "attribute",
            "display_name",
            "display_info",
            "potential_youtube_reach",
            "subscriber_share",
            "viewer_share",
            "youtube_channel_metadata",
            "youtube_video_metadata",
            "dynamic_attribute_metadata",
            "location_attribute_metadata",
            "user_interest_attribute_metadata",
            "knowledge_graph_attribute_metadata",
        ],
    ) -> bool: ...

class AudienceInsightsCategory(proto.Message):
    category_id: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        category_id: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["category_id"]
    ) -> bool: ...

class AudienceInsightsDynamicLineup(proto.Message):
    dynamic_lineup_id: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        dynamic_lineup_id: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["dynamic_lineup_id"]
    ) -> bool: ...

class AudienceInsightsEntity(proto.Message):
    knowledge_graph_machine_id: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        knowledge_graph_machine_id: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["knowledge_graph_machine_id"]
    ) -> bool: ...

class AudienceInsightsTopic(proto.Message):
    entity: AudienceInsightsEntity
    category: AudienceInsightsCategory
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        entity: AudienceInsightsEntity = ...,
        category: AudienceInsightsCategory = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["entity", "category"]
    ) -> bool: ...

class DynamicLineupAttributeMetadata(proto.Message):
    class SampleChannel(proto.Message):
        youtube_channel: YouTubeChannelInfo
        display_name: str
        youtube_channel_metadata: YouTubeChannelAttributeMetadata
        def __init__(
            self: _M,
            mapping: _M | Mapping | google.protobuf.message.Message | None = None,
            *,
            ignore_unknown_fields: bool = False,
            youtube_channel: YouTubeChannelInfo = ...,
            display_name: str = ...,
            youtube_channel_metadata: YouTubeChannelAttributeMetadata = ...,
        ) -> None: ...
        def __contains__(  # type: ignore[override]
            self,
            key: Literal["youtube_channel", "display_name", "youtube_channel_metadata"],
        ) -> bool: ...

    inventory_country: LocationInfo
    median_monthly_inventory: int
    channel_count_lower_bound: int
    channel_count_upper_bound: int
    sample_channels: MutableSequence[DynamicLineupAttributeMetadata.SampleChannel]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        inventory_country: LocationInfo = ...,
        median_monthly_inventory: int = ...,
        channel_count_lower_bound: int = ...,
        channel_count_upper_bound: int = ...,
        sample_channels: MutableSequence[
            DynamicLineupAttributeMetadata.SampleChannel
        ] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal[
            "inventory_country",
            "median_monthly_inventory",
            "channel_count_lower_bound",
            "channel_count_upper_bound",
            "sample_channels",
        ],
    ) -> bool: ...

class KnowledgeGraphAttributeMetadata(proto.Message):
    entity_capabilities: MutableSequence[
        InsightsKnowledgeGraphEntityCapabilitiesEnum.InsightsKnowledgeGraphEntityCapabilities
    ]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        entity_capabilities: MutableSequence[
            InsightsKnowledgeGraphEntityCapabilitiesEnum.InsightsKnowledgeGraphEntityCapabilities
        ] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["entity_capabilities"]
    ) -> bool: ...

class LocationAttributeMetadata(proto.Message):
    country_location: LocationInfo
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        country_location: LocationInfo = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["country_location"]
    ) -> bool: ...

class UserInterestAttributeMetadata(proto.Message):
    user_interest_description: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        user_interest_description: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["user_interest_description"]
    ) -> bool: ...

class YouTubeChannelAttributeMetadata(proto.Message):
    subscriber_count: int
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        subscriber_count: int = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["subscriber_count"]
    ) -> bool: ...

class YouTubeVideoAttributeMetadata(proto.Message):
    thumbnail_url: str
    video_url: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        thumbnail_url: str = ...,
        video_url: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["thumbnail_url", "video_url"]
    ) -> bool: ...



from collections.abc import Mapping, MutableSequence
from typing import Any, NoReturn, TypeVar

import google.protobuf.message
import proto
from google.protobuf.field_mask_pb2 import FieldMask
from google.rpc.status_pb2 import Status
from typing_extensions import Literal

from google.ads.googleads.v19.resources.types.keyword_plan_campaign_keyword import (
    KeywordPlanCampaignKeyword,
)

_M = TypeVar("_M")

class KeywordPlanCampaignKeywordOperation(proto.Message):
    update_mask: FieldMask
    create: KeywordPlanCampaignKeyword
    update: KeywordPlanCampaignKeyword
    remove: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        update_mask: FieldMask = ...,
        create: KeywordPlanCampaignKeyword = ...,
        update: KeywordPlanCampaignKeyword = ...,
        remove: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["update_mask", "create", "update", "remove"]
    ) -> bool: ...

class MutateKeywordPlanCampaignKeywordResult(proto.Message):
    resource_name: str
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        resource_name: str = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["resource_name"]
    ) -> bool: ...

class MutateKeywordPlanCampaignKeywordsRequest(proto.Message):
    customer_id: str
    operations: MutableSequence[KeywordPlanCampaignKeywordOperation]
    partial_failure: bool
    validate_only: bool
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        customer_id: str = ...,
        operations: MutableSequence[KeywordPlanCampaignKeywordOperation] = ...,
        partial_failure: bool = ...,
        validate_only: bool = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self,
        key: Literal["customer_id", "operations", "partial_failure", "validate_only"],
    ) -> bool: ...

class MutateKeywordPlanCampaignKeywordsResponse(proto.Message):
    partial_failure_error: Status
    results: MutableSequence[MutateKeywordPlanCampaignKeywordResult]
    def __init__(
        self: _M,
        mapping: _M | Mapping | google.protobuf.message.Message | None = None,
        *,
        ignore_unknown_fields: bool = False,
        partial_failure_error: Status = ...,
        results: MutableSequence[MutateKeywordPlanCampaignKeywordResult] = ...,
    ) -> None: ...
    def __contains__(  # type: ignore[override]
        self, key: Literal["partial_failure_error", "results"]
    ) -> bool: ...
